// vi: set ts=4 sw=4 :
// vim: set tw=75 :
#include <extdll.h>
/*
* Copyright(c)2001 Will Day <willday@hpgx.net>
*
* This file is part of Metamod.
*
* Metamod is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or(at
* your option)any later version.
*
* Metamod is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Metamod; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* In addition, as a special exception, the author gives permission to
* link the code of this program with the Half-Life Game Engine("HL
* Engine")and Modified Game Libraries("MODs")developed by Valve,
* L.L.C("Valve"). You must obey the GNU General Public License in all
* respects for all of the code used other than the HL Engine and MODs
* from Valve. If you modify this file, you may extend this exception
* to your version of the file, but you are not obligated to do so. If
* you do not wish to do so, delete this exception statement from your
* version.
*
*/
#include <dllapi.h>
#include <meta_api.h>
#include "usercmd.h"
extern "C"
{
#include "pm_shared.h"
}
#include "enginecallback.h"
#include "sdk_util.h"
#include "cbase.h"
#include "const.h"
#include "weaponinfo.h"
#include "progdefs.h"
#include "scriplib.h"
#include "axn_plugin.h"
#include "cstrike.h"
#include "entity.h"

#include "usrmsg.h"


extern globalvars_t *gpGlobals;

cvar_t cvar_axn_version	=	{"axn_version",AXN_VERSION, FCVAR_SERVER };
cvar_t cvar_showaddress	=	{"axn_showaddress","0"};
cvar_t cvar_tracer		=	{"axn_tracer","0"};
cvar_t cvar_starthealth	=	{"axn_starthealth","100"};
cvar_t cvar_statustext	=	{"axn_statustext","0"};
cvar_t cvar_noheadshot	=	{"axn_noheadshot","0"};
cvar_t cvar_flashtime	=	{"axn_flashtime","1"};
cvar_t cvar_smokeradius	=	{"axn_smokeradius","0"};
cvar_t cvar_smokegasp	=	{"axn_smokegasp","0"};
cvar_t cvar_damageimpact=	{"axn_damageimpact","0"};
cvar_t cvar_longjump	=	{"axn_longjump","0"};
cvar_t cvar_bunnyjump	=	{"axn_bunnyjump","0"};
cvar_t cvar_jumpability	=	{"axn_jumpability","1"};



cvar_t *axn_version=NULL;

cvar_t *axn_showaddress=NULL;
cvar_t *axn_tracer=NULL;
cvar_t *axn_starthealth=NULL;
cvar_t *axn_statustext=NULL;
cvar_t *axn_noheadshot=NULL;
cvar_t *axn_damageimpact=NULL;
cvar_t *axn_longjump=NULL;
cvar_t *axn_bunnyjump=NULL;
cvar_t *axn_jumpability=NULL;
cvar_t *axn_flashtime=NULL;

cvar_t *axn_smokeradius=NULL;
cvar_t *axn_smokegasp=NULL;


edict_t *listenserver_edict=NULL;


void GameDLLInit(void)
{
	// ALERT(at_console,"GameDLLInit\n");
	CVAR_REGISTER(&cvar_axn_version);
	CVAR_REGISTER(&cvar_showaddress);
	CVAR_REGISTER(&cvar_tracer);
	CVAR_REGISTER(&cvar_starthealth);
	CVAR_REGISTER(&cvar_statustext);
	CVAR_REGISTER(&cvar_noheadshot);
	CVAR_REGISTER(&cvar_damageimpact);
	CVAR_REGISTER(&cvar_longjump);
	CVAR_REGISTER(&cvar_bunnyjump);
	CVAR_REGISTER(&cvar_jumpability);
	CVAR_REGISTER(&cvar_flashtime);


	axn_version	=	CVAR_GET_POINTER("axn_version");
	axn_showaddress	=	CVAR_GET_POINTER("axn_showaddress");
	axn_tracer	=	CVAR_GET_POINTER("axn_tracer");
	axn_starthealth	=	CVAR_GET_POINTER("axn_starthealth");
	axn_statustext	=	CVAR_GET_POINTER("axn_statustext");
	axn_noheadshot	=	CVAR_GET_POINTER("axn_noheadshot");
	axn_damageimpact=	CVAR_GET_POINTER("axn_damageimpact");
	axn_longjump	=	CVAR_GET_POINTER("axn_longjump");
	axn_bunnyjump	=	CVAR_GET_POINTER("axn_bunnyjump");
	axn_jumpability	=	CVAR_GET_POINTER("axn_jumpability");
	axn_flashtime	=	CVAR_GET_POINTER("axn_flashtime");


	if(g_mod_type==CSTRIKE)
	{

		CVAR_REGISTER(&cvar_smokeradius);
		CVAR_REGISTER(&cvar_smokegasp);
		axn_smokeradius	=	CVAR_GET_POINTER("axn_smokeradius");
		axn_smokegasp	=	CVAR_GET_POINTER("axn_smokegasp");

	}

	RETURN_META(MRES_IGNORED);
}
int DispatchSpawn(edict_t *pent)
{
	int i=0;
	char *pClassname=(char *)STRING(pent->v.classname);
	// ALERT(at_console,"DispatchSpawn %s\n",pClassname);
	if(strcmp(pClassname, "worldspawn")==0)
	{
		// do level initialization stuff here...
		PRECACHE_MODEL("models/w_oxygen.mdl");
		PRECACHE_MODEL("models/w_medkit.mdl");
		PRECACHE_MODEL("models/can.mdl");
		PRECACHE_MODEL("sprites/laserbeam.spr");
		PRECACHE_MODEL("models/office_gibs.mdl");
		PRECACHE_SOUND("fvox/bell.wav");
		PRECACHE_MODEL("models/scientist.mdl");
		if(g_mod_type==CSTRIKE)
		{
			PRECACHE_MODEL("models/hostage.mdl");
			PRECACHE_SOUND("hostage/hos1.wav");
			PRECACHE_SOUND("hostage/hos2.wav");
			PRECACHE_SOUND("hostage/hos3.wav");
			PRECACHE_SOUND("hostage/hos4.wav");
			PRECACHE_SOUND("hostage/hos5.wav");
			PRECACHE_SOUND("player/gasp1.wav");
			PRECACHE_SOUND("player/gasp2.wav");
		}
		for(i=0;i<num_entities;i++)
		{
			Precache(&entities[i]);
		}
		for(i=0;i<num_entities;i++)
		{
			entities[i].index=AddEntity(&entities[i]);
		}

	}
	for(i=0;i<num_rements;i++)
	{
		if(strcmp(pClassname, rements[i])==0&&strcmp(pClassname,"worldspawn")!=0)
		{
			// ALERT(at_console,"Remove %s\n",pClassname);
			REMOVE_ENTITY(pent);
			RETURN_META_VALUE(MRES_SUPERCEDE, 0);
		}
	}
	RETURN_META_VALUE(MRES_IGNORED, META_RESULT_PREVIOUS);
}
void DispatchKeyValue(edict_t *pentKeyvalue, KeyValueData *pkvd)
{
	// ALERT(at_console,"DispatchKeyValue %s %s\n",pkvd->szKeyName,pkvd->szValue);
	if(pkvd&&pkvd->szValue)
	{
		if(strcmp(pkvd->szValue,"worldspawn")==0)
		{
			bSpawn=true;
			RemoveEntities();
			LoadEntities();
		}
	}
	RETURN_META(MRES_IGNORED);
}
BOOL ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ])
{
	// ALERT(at_console,"ClientConnect\n");
	if(strcmp(pszAddress, "loopback")==0)
	{
		// save the edict of the listen server client...
		listenserver_edict=pEntity;
	}

	int i=ENTINDEX(pEntity);
	memset(&g_PlayerInfo[i],0,sizeof(player_extra_info_t));
	sprintf(g_PlayerInfo[i].address,"%s",pszAddress);
//	UTIL_GetHostName(g_PlayerInfo[i].address);


	if(axn_showaddress->value>0)
	{
		UTIL_SayTextAll(UTIL_VarArgs("%s(%d@%s) is trying to connect.\n",pszName,g_engfuncs.pfnGetPlayerWONId(pEntity),g_PlayerInfo[i].address),pEntity);
	}

	RETURN_META_VALUE(MRES_IGNORED,META_RESULT_PREVIOUS);
}
void ClientDisconnect(edict_t *pEntity)
{
	// ALERT(at_console,"ClientDisconnect\n");
	int i=ENTINDEX(pEntity);
	memset(&g_PlayerInfo[i],0,sizeof(player_extra_info_t));

	RETURN_META(MRES_IGNORED);
}
void ClientCommand(edict_t *pEntity)
{
	// ALERT(at_console,"ClientCommand %s\n",pcmd);
	if(check_cmd(pEntity))
		RETURN_META(MRES_SUPERCEDE);
	RETURN_META(MRES_IGNORED);
}
void PlayerPreThink(edict_t *pEntity)
{
	// ALERT(at_console,"PlayerPreThink\n");
	if(g_mod_type==CSTRIKE)CS_PlayerThink(pEntity);

	RETURN_META(MRES_IGNORED);
}


void DispatchPM_Move(struct playermove_s *ppmove, int server)
{
	if(axn_longjump->value==0&&axn_bunnyjump->value==0&&axn_jumpability->value>=1)RETURN_META(MRES_IGNORED);
	PM_Move(ppmove, server);
	RETURN_META(MRES_SUPERCEDE);
}
void DispatchPM_Init(struct playermove_s *ppmove)
{
	PM_Init(ppmove);
	RETURN_META(MRES_IGNORED);
}


static DLL_FUNCTIONS gFunctionTable=
{
	NULL,//GameDLLInit, // pfnGameInit
	DispatchSpawn, // pfnSpawn
	NULL, // pfnThink
	NULL, // pfnUse
	NULL, // pfnTouch
	NULL, // pfnBlocked
	DispatchKeyValue, // pfnKeyValue
	NULL, // pfnSave
	NULL, // pfnRestore
	NULL, // pfnSetAbsBox
	NULL, // pfnSaveWriteFields
	NULL, // pfnSaveReadFields
	NULL, // pfnSaveGlobalState
	NULL, // pfnRestoreGlobalState
	NULL, // pfnResetGlobalState
	ClientConnect, // pfnClientConnect
	ClientDisconnect, // pfnClientDisconnect
	NULL, // pfnClientKill
	NULL, // pfnClientPutInServer
	ClientCommand, // pfnClientCommand
	NULL, // pfnClientUserInfoChanged
	NULL, // pfnServerActivate
	NULL, // pfnServerDeactivate
	PlayerPreThink, // pfnPlayerPreThink
	NULL, // pfnPlayerPostThink
	NULL, // pfnStartFrame
	NULL, // pfnParmsNewLevel
	NULL, // pfnParmsChangeLevel
	NULL, // pfnGetGameDescription
	NULL, // pfnPlayerCustomization
	NULL, // pfnSpectatorConnect
	NULL, // pfnSpectatorDisconnect
	NULL, // pfnSpectatorThink
	NULL, // pfnSys_Error
	DispatchPM_Move, // pfnPM_Move
	DispatchPM_Init, // pfnPM_Init
	NULL, // pfnPM_FindTextureType
	NULL, // pfnSetupVisibility
	NULL, // pfnUpdateClientData
	NULL,//AddToFullPack, // pfnAddToFullPack
	NULL, // pfnCreateBaseline
	NULL, // pfnRegisterEncoders
	NULL, // pfnGetWeaponData
	NULL, // pfnCmdStart
	NULL, // pfnCmdEnd
	NULL, // pfnConnectionlessPacket
	NULL, // pfnGetHullBounds
	NULL, // pfnCreateInstancedBaselines
	NULL, // pfnInconsistentFile
	NULL, // pfnAllowLagCompensation
};
C_DLLEXPORT int GetEntityAPI2(DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion)
{
	if(!pFunctionTable){
		UTIL_LogPrintf("GetEntityAPI2 called with null pFunctionTable");
		return(FALSE);
	}
	else if(*interfaceVersion!=INTERFACE_VERSION){
		UTIL_LogPrintf("GetEntityAPI2 version mismatch; requested=%d ours=%d", *interfaceVersion, INTERFACE_VERSION);
		//! Tell engine what version we had, so it can figure out who is out of date.
		*interfaceVersion=INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy(pFunctionTable, &gFunctionTable, sizeof(DLL_FUNCTIONS));
	return(TRUE);
}
